#include "EpicSummary.h"


EpicSummary::~EpicSummary()
{
	/* Need this to stop destructor destroying input data sets with the object */
	_fitsum = NULL;
	_pidx = NULL;

}

EpicSummary::EpicSummary(DataSet* fitsum , DataSet* pidx ) 
: PpssummCommon( fitsum , pidx )  
{
	/* The main contents */

	multi_exposure_title();
	multi_exposure_products(); 
	
	exposures_processed();

	exposure_specific_title();
	exposure_specific_products();

}

void EpicSummary::multi_exposure_title()
{
	multi_title.clear();
	multi_title.push_back(colName("FILENAME"));
	multi_title.push_back(colName("CONTENT"));
}

void EpicSummary::multi_exposure_products()
{
	errstr << "Entering multi_exposure_products" << Sas::message(Msg::App, Msg::Sparse);

	multi_table.clear();

	if ( _pidx->hasBlock("EPICOBS") ) {
	
		multi_table.insert(multi_table.begin() , 2 , strlist() );
		strlist tmpf , tmpc;

		Table* tbl = _pidx->table("EPICOBS");

		if (tbl->hasColumn("NAME")) {

			Column* col = tbl->column("NAME");
			stringColumn(col , back_inserter(tmpf) );
			std::transform( tmpf.begin()
				, tmpf.end() 
				, tmpf.begin() 
				, _toString<std::string>() 
			);
		}
		
		if (tbl->hasColumn("CONTENT")) {
			Column* col = tbl->column("CONTENT");
			stringColumn(col , back_inserter(tmpc) );
			std::transform( tmpc.begin() 
				, tmpc.end() 
				, tmpc.begin() 
				, _toString<std::string>() 
			);
		}

		/* Remove what we don't want.
		 * Filter on content first
		 */
	
		for ( strlist::size_type offset=0 ; offset < tmpc.size() ; offset++ ) {

			if  ( tmpc[offset].find("OBSERVATION SUMMARY") != std::string::npos)
				continue;

			multi_table[0].push_back ( tmpf[offset] );
			multi_table[1].push_back ( tmpc[offset] );

		}

		/* Sort what is left */
		if ( multi_table[0].size() && multi_table[1].size() ) {
			
			vector< strlist::difference_type > dt;
			sort_order( multi_table[1].begin()
				,multi_table[1].end()
				,back_inserter(dt)
			);

			reorder( multi_table[0].begin()
				,multi_table[0].end()
				,dt.begin()
				,dt.end()
			);
		} else {
			multi_table.clear();
		}

		
	}
	
	errstr << "Leaving multi_exposure_products" << Sas::message(Msg::App, Msg::Sparse);
	
}

void EpicSummary::exposures_processed()
{
	errstr << "Entering exposures_processed" << Sas::message(Msg::App, Msg::Sparse);
	/* 
	 * We should already have all necessay exposure data in PpssummCommon::_processed_tables.
	 *
	 * As usual, things are never simple.  Just because an instrument is active does not
	 * mean any exposures were processed.  If no exposures were processed we won't find
	 * the associated table in _pidx.
	 */

	/* Bad hack, copied from PpssummCommon */
	
        enum ExpColId { 
                INSTRUMENT =0 
                ,EXPID    
                ,SCHEDULED 
                ,MODE      
                ,DATA_MODE 
                ,FILTER   
                ,FILTER_WHEEL  // PN Uses column name FILTER_POTENTIOMETER
                ,DURATION
                ,ACTUAL_START
                ,ACTUAL_END  
                ,NUMCOL
        };

	exp_table.clear();

	std::string instruments[] = { "M1" , "M2" , "PN" , "" };

	for (std::string* inst = instruments ; inst->size() ; inst++ ) {

		keyed_tables::const_iterator PI = _processed_tables.find( *inst );

		if ( PI != _processed_tables.end() ) {
			
			const strtable& idata = PI->second;

			if ( ! exp_table.size() ) exp_table.insert( exp_table.begin() , NUMCOL , strlist() );
			
			for ( unsigned int i = 0 ; i < idata.size() ; i++ ) {

				exp_table[i].insert( exp_table[i].end() , idata[i].begin() , idata[i].end() );
			}
		}	
	}
	errstr << "Leaving exposures_processed" << Sas::message(Msg::App, Msg::Sparse);
	
}

void EpicSummary::exposure_specific_title()
{

	specific_title.clear();
	specific_title.push_back(colName("FILENAME"));
	specific_title.push_back(colName("CONTENT"));
	specific_title.push_back(colName("BAND"));
	specific_title.push_back(colName("VVFLAGS"));

}

void EpicSummary::exposure_specific_products()
{
	typedef pair< strlist::difference_type, strlist::difference_type > offset_range; 

	errstr << "Entering exposure_specific_products" << Sas::message(Msg::App, Msg::Sparse);
	/* That bad hack again */

        enum ExpColId { 
                INSTRUMENT =0 
                ,EXPID    
                ,SCHEDULED 
                ,MODE      
                ,DATA_MODE 
                ,FILTER   
                ,FILTER_WHEEL  // PN Uses column name FILTER_POTENTIOMETER
                ,DURATION
                ,ACTUAL_START
                ,ACTUAL_END  
                ,NUMCOL
        };

	
	enum SpecColId {
		NAME = 0
		,CONTENT
		,BAND
		,VVFLAGS
		,NUMSPECCOL
	};

	const std::string instruments[] = { 
		"EMOS1" 
		, "EMOS2" 
		, "EPN" 
		, "" 
	};

	/* 
	 * _pidx tables exposure spcific data comes from
	 * These tables are mixed RGS1 and RGS2 so we need to do the
	 * first selection based on intrument, then exposure id
	 */			
	const std::string tables[] = { 
		  "EPICEXP" 
		, "" 
	};

	/* The normal extra columns we want and which column they go into */
	const std::string want[] = {  
		  "NAME" 
		, "CONTENT" 
		, "VVFLAGS" 
		, "" 
	};
	
	const unsigned int to[] = { NAME , CONTENT , VVFLAGS  };

	/*
	 * Special columns 
	 * BAND is int32 and is not in all tables and are handled separately 
	 */

	
	for (const std::string* inst = instruments ; inst->size() ; inst++ ) {

		keyed_tables::const_iterator PI = _processed_tables.find( PpssummCommon::instName(*inst) );

		if ( PI == _processed_tables.end() ) continue;
	
		for(const std::string* table =tables ; table->size() ; table++ ) {
			
			if (! _pidx->hasBlock(*table) ) continue;

			Table* tbl = _pidx->table(*table);
			Column* col;
		
			/* First select on instrument */
			col = tbl->column("INSTRUMENT");

			strlist foundinst;
			stringColumn( col , back_inserter( foundinst ) );
			std::transform( foundinst.begin() 
				, foundinst.end() 
				, foundinst.begin() 
				, _toString<std::string>() 
			);
			

			vector<strlist::difference_type> inst_rows;
			strlist::const_iterator Ib =  foundinst.begin();
			strlist::const_iterator Ie =  foundinst.end();


			/* Get all rows matching the current instrument */
			find_all( Ib 
				, Ie 
				, *inst 
				, back_inserter(inst_rows)
			);

			std::sort(inst_rows.begin() , inst_rows.end());
			std::unique(inst_rows.begin() , inst_rows.end());

			/* skip if we didn't find in this table */
			if ( ! inst_rows.size() ) continue;

			/* 
			 * We want all of the exposure ids for this instrument 
			 * Sort this column and reorder the instrument rows
			 * accordingly.
			 * Then all the rows for an exposure will be given by an
			 * range of the instrument rows defined by an equal_range of
			 * the exposure rows.
			 *
			 * */
			strlist foundexp;
			col = tbl->column("EXP_ID");

			stringColumn( col , back_inserter(foundexp) , inst_rows.begin() , inst_rows.end() );
			std::transform( foundexp.begin() , foundexp.end() , foundexp.begin() , _toString<std::string>() );

			std::vector<strlist::difference_type> exp_pos;
			sort_order( foundexp.begin() , foundexp.end() , back_inserter(exp_pos) );

			reorder(inst_rows.begin() , inst_rows.end() , exp_pos.begin() , exp_pos.end() );

                	vector< offset_range > offsetranges;

                	equal_ranges(foundexp.begin() , foundexp.end() , back_inserter(offsetranges));

			for ( vector< offset_range >::const_iterator offset = offsetranges.begin()
				; offset != offsetranges.end()
				; offset ++ 
			) {

			
				std::string expid = *(foundexp.begin() + offset->first);
				vector<strlist::difference_type> rows( inst_rows.begin() + offset->first
					, inst_rows.begin() + offset->second 
				);
				
						
				/* Now read the rows of relevant columns */
				std::string key= *inst + expid;
				strtable& idata = specific_table[key];
				if ( ! idata.size() ) idata.insert( idata.begin(), NUMSPECCOL , strlist() );

				/* 
				 * Read these columns from _pidx retrieving only those refering to the
				 * instrument we are after
				 */

				for (unsigned int i=0; i < 3 ; i++) { 
			

					col = tbl->column(want[i]);
					stringColumn( col 
						,  back_inserter(idata[to[i]]) 
						, rows.begin() 
						, rows.end() 
					);
					std::transform( idata[to[i]].begin() 
						, idata[to[i]].end() 
						, idata[to[i]].begin() 
						, _toString<std::string>() 
					);
				}

				// BAND is int32 and the 4th column
				col = tbl->column("BAND");
				vector<int32> vi;
				int32Column( col 
					, back_inserter(vi) 
					, rows.begin() 
					, rows.end() 
				);

				std::transform( vi.begin() 
					, vi.end() 
					, back_inserter(idata[BAND])
					, _toString<int32>()
				);
				
			}
		}
	}

	for (keyed_tables::iterator CI = specific_table.begin()
		; CI != specific_table.end()
		; CI++
	) {

		PpssummStatic::_anchors.push_back(CI->first);
		strtable& idata = CI->second;

		/* Sort columns on band before conversion to energy values */
		vector< strlist::difference_type > newsort;
		sort_order( idata[BAND].begin()
			, idata[BAND].end()
			, back_inserter(newsort)
		);

		for ( strlist::iterator I = idata[BAND].begin()
			; I !=  idata[BAND].end() 
			; I++
		) {
			*I = PpssummStatic::band(*I);
		}

		/* re-order the rest of the columns*/
		for (unsigned int i = 0 ; i < 3 ; i++) {
			reorder( idata[to[i]].begin()
				, idata[to[i]].end()
				, newsort.begin()
				, newsort.end()
			);
		};
	}

		
	errstr << "Leaving exposure_specific_products" << Sas::message(Msg::App, Msg::Sparse);
	
}

HtmlPage EpicSummary::print() 
{
	HtmlPage _page("epxpage");

	std::string htitle("Observation " + PpssummCommon::_obsid + " Pipeline Processing Summary");
	errstr << "Generating " << htitle << Sas::message(Msg::App, Msg::Verbose);
	const HtmlTag hr("hr");

	/* Top of the page */ 
	_page.title(htitle) ;

	_page.linkbar("epxpage");
	_page._body << hr ;
	
	
	_page.sub_title(PpssummCommon::_sub_title , PpssummCommon::_obsid + " EPIC Processing Summary");
	_page._body << hr ;

	/* Middle of page */
	print_multi_exposure_products(_page);
	print_exposures_processed(_page);
	print_exposure_specific_products(_page);
	
	/* end of page */


	return _page;	

}

void EpicSummary::print_multi_exposure_products(HtmlPage& page)
{
	page._body << HtmlTag("h3" , "" , "EPIC Multi-Exposure Products");
	if ( multi_table.size() ) {
		
		page.addTable(multi_title , multi_table);

	} else {
		page._body << HtmlTag("tt","","No multi-exposure products were produced.");
	}
	page._body  << HtmlTag("hr");

}

void EpicSummary::print_exposures_processed(HtmlPage& page)
{
	page._body << HtmlTag("h3","","Exposures processed by PPS");

	if (exp_table.size() ) {

		page.addTable( PpssummCommon::_exposure_title , exp_table );

	} else {

		page._body << HtmlTag("tt","","No EPIC exposures processed by the PPS.");

	}
	page._body  << HtmlTag("hr");

}

void EpicSummary::print_exposure_specific_products(HtmlPage& page) 
{
	page._body << HtmlTag("h3","","Exposure Specific Products");

	if (exp_table.size() && specific_table.size() ) {

		/* Loop over exp_table so we get specific products in the same order */
		for (unsigned int i = 0 ; i < exp_table[0].size() ; i++ ) {
	
			std::string key =  exp_table[0][i] + exp_table[1][i];
			keyed_tables::iterator MI = specific_table.find( key );

			/* ignore exposures where there are no specific products */
			if ( MI ==  specific_table.end() ) continue;

			/* Copy the exposure information into a separate table for output */
			strtable exptmp(exp_table.size());
			for (unsigned int j = 0 ; j < exp_table.size() ; j++) {
				exptmp[j] = strlist(1 , exp_table[j][i]);
			}
			
			page.addTable( PpssummCommon::_exposure_title 
				, exptmp 
				, key
			);
			page.addTable( specific_title , MI->second );
			page._body << HtmlTag("hr");
		}

	} else {
		page._body << HtmlTag("tt","","No exposure specific products were produced.");
		page._body  << HtmlTag("hr");

	}

}
