#ifndef __PPSSUMM_EXTENDFITSCOLUMN__
#define __PPSSUMM_EXTENDFITSCOLUMN__

/* 
 *
 * Extending the functions provided by FitsColumn.h
 * with an alternative interface.
 *
 */

#include <iterator>
#include "ppssumm_algo.h"
#include "FitsColumn.h"
#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"

template< typename _Oi  >
_Oi boolColumn( const Column* col, _Oi _result , unsigned int from = 0, unsigned int nr = 0  ) 
{

	unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
	vector<unsigned int> v;
	Tseq( from  , rows , back_inserter(v));
	return boolColumn(col , _result , v.begin() , v.end() );

}

template< typename _Oi  >
_Oi int8Column( const Column* col, _Oi _result , unsigned int from = 0 , unsigned int nr = 0 ) 
{

	unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
	vector<unsigned int> v;
	Tseq( from  , rows , back_inserter(v));
	return int8Column(col , _result , v.begin() , v.end() );

}

template< typename _Oi  >
_Oi uint16Column( const Column* col, _Oi _result , unsigned int from = 0, unsigned int nr = 0  ) 
{

	unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
	vector<unsigned int> v;
	Tseq( from  , rows , back_inserter(v));
	return uint16Column(col , _result , v.begin() , v.end() );

}

template< typename _Oi  >
_Oi int16Column( const Column* col, _Oi _result , unsigned int from = 0, unsigned int nr = 0 ) 
{

	unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
	vector<unsigned int> v;
	Tseq( from  , rows , back_inserter(v));
	return int16Column(col , _result , v.begin() , v.end() );

}

template< typename _Oi  >
_Oi uint32Column( const Column* col, _Oi _result , unsigned int from = 0, unsigned int nr = 0 ) 
{

	unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
	vector<unsigned int> v;
	Tseq( from  , rows , back_inserter(v));
	return uint32Column(col , _result , v.begin() , v.end() );

}

template< typename _Oi  >
_Oi int32Column( const Column* col, _Oi _result , unsigned int from = 0, unsigned int nr = 0 ) 
{

	unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
	vector<unsigned int> v;
	Tseq( from  , rows , back_inserter(v));
	return int32Column(col , _result , v.begin() , v.end() );

}

template< typename _Oi  >
_Oi real32Column( const Column* col, _Oi _result , unsigned int from = 0, unsigned int nr = 0 ) 
{

	unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
	vector<unsigned int> v;
	Tseq( from  , rows , back_inserter(v));
	return real32Column(col , _result , v.begin() , v.end() );

}

template< typename _Oi  >
_Oi real64Column( const Column* col, _Oi _result , unsigned int from = 0, unsigned int nr = 0 ) 
{

	unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
	vector<unsigned int> v;
	Tseq( from  , rows , back_inserter(v));
	return real64Column(col , _result , v.begin() , v.end() );

}

template< typename _Oi >
_Oi stringColumn( const Column* col , _Oi _result , unsigned int from = 0 , unsigned int nr = 0  ) 
{

	unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
	vector<unsigned int> v;
	Tseq( from  , rows , back_inserter(v));
	return stringColumn(col , _result , v.begin() , v.end() );

}


#endif
