#ifndef __PPSSUMM_FITSCOLUMN__
#define __PPSSUMM_FITSCOLUMN__

/* 
 *
 * Functions for reading FIT columns via DAL
 * into an iterator via a template function
 * Rows to read are provided by an iterator sequence
 * of unsigned integers
 *
 * No conversion is done here so the output has to be fo the right type!
 *
 */

#include <string>
#include <iostream>
#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"

template< typename _Oi , typename _Ii >
_Oi boolColumn( const Column* col, _Oi _result ,  _Ii _first , _Ii _last ) 
{
	ColumnData *coldat = col->data();
	bool8* data = coldat->boolData();
	while (_first != _last) {
		
		if ( (*_first) < col->parent()->rows() ) {
			*_result++ = data[*_first] ;
		} else {
			errstr << "bool8Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
		}
		_first++;
	}
	delete (coldat);
	return _result;
}

template< typename _Oi , typename _Ii >
_Oi int8Column( const Column* col , _Oi _result , _Ii _first , _Ii _last )
{
	ColumnData *coldat = col->data();
	int8* data = coldat->int8Data();
	while (_first != _last) {
		
		if ( static_cast<unsigned int>(*_first) < col->parent()->rows()) {
			*_result++ =  data[*_first] ;
		} else {
			errstr << "int8Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
		}
		_first++;
	}
	delete (coldat);
	return _result;
}

template< typename _Oi , typename _Ii >
_Oi uint16Column( const Column* col, _Oi _result , _Ii _first, _Ii _last ) 
{
	ColumnData *coldat = col->data();
	uint16* data = coldat->uint16Data();
	while (_first != _last) {
		
		if ( static_cast<unsigned int>(*_first) < col->parent()->rows()) {
			*_result++ =  data[*_first] ;
		} else {
			errstr << "uint16Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
		}
		_first++;
	}
	delete (coldat);
	return _result;
}


template< typename _Oi , typename _Ii >
_Oi int16Column( const Column* col , _Oi _result , _Ii _first , _Ii _last ) 
{
	ColumnData *coldat = col->data();
	int16* data = coldat->int16Data();
	while (_first != _last) {
		
		if ( static_cast<unsigned int>(*_first) < col->parent()->rows()) {
			*_result++ =  data[*_first] ;
		} else {
			errstr << "int16Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
		}
		_first++;
	}
	delete (coldat);
	return _result;
}

template< typename _Oi , typename _Ii >
_Oi uint32Column( const Column* col , _Oi _result , _Ii _first , _Ii _last ) 
{
	ColumnData *coldat = col->data();
	uint32* data = coldat->uint32Data();
	while (_first != _last) {
		
		if ( static_cast<unsigned int>(*_first) < col->parent()->rows()) {
			*_result++ =  data[*_first] ;
		} else {
			errstr << "uint32Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
		}
		_first++;
	}
	delete (coldat);
	return _result;
}


template< typename _Oi , typename _Ii >
_Oi int32Column( const Column* col, _Oi _result , _Ii _first , _Ii _last ) 
{
	ColumnData *coldat = col->data();
	int32* data = coldat->int32Data();
	while (_first != _last) {
		
		if (static_cast<unsigned int>(*_first) < col->parent()->rows()) {
			*_result++ =  data[*_first] ;
		} else {
			errstr << "int32Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
		}
		_first++;
	}
	delete (coldat);
	return _result;
}

template< typename _Oi , typename _Ii >
_Oi real32Column( const Column* col, _Oi _result ,_Ii _first, _Ii _last ) 
{
	ColumnData *coldat = col->data();
	real32* data = coldat->real32Data();
	while (_first != _last) {
		
		if ( static_cast<unsigned int>(*_first) < col->parent()->rows()) {
			*_result++ =  data[*_first] ;
		} else {
			errstr << "real32Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
		}
		_first++;
	}
	delete (coldat);
	return _result;
}

template< typename _Oi , typename _Ii >
_Oi real64Column( const Column* col, _Oi _result , _Ii _first , _Ii _last ) 
{
	ColumnData *coldat = col->data();
	real64* data = coldat->real64Data();
	while (_first != _last) {
		
		if ( static_cast<unsigned int>(*_first) < col->parent()->rows()) {
				
			*_result++ =  data[*_first] ;
		} else {
			errstr << "real64Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
		}
		_first++;
	}
	delete (coldat);
	return _result;
}

template< typename _Oi , typename _Ii >
_Oi stringColumn( const Column* col , _Oi _result , _Ii _first , _Ii _last ) 
{
	unsigned int width = col->elements();

	ColumnData *coldat = col->data();
	char* data = coldat->stringData();
	while (_first != _last) {

		if (static_cast<unsigned int>(*_first) < col->parent()->rows()) {
			*_result++ = std::string(data + (*_first)*width , data + (*_first+1)*width) ;
		} else { 
			errstr << "stringColumn: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
		}
		_first++;
		
	}
	delete (coldat);
	return _result;
}



#endif
