#ifndef __PPSSUMM_HTMLTAG__
#define __PPSSUMM_HTMLTAG__

#include <iostream>
#include <sstream>
#include <string>

/* The contents between the tags are stored in this*/
using namespace std;

class HtmlTag
: public string
{
	public:
	HtmlTag()
	: string("") , _tag("") , _extra("") , _pre("") , _post("") 
	{  }

	HtmlTag(string t, string e="" , string c="" , string pre="" , string post="")
	: string(c) , _tag(t) , _extra(e) , _pre(pre) , _post(post) 
	{  }

	HtmlTag(string t, string e , HtmlTag ht , string pre="" , string post="" ) 
	: string( ht.print() ) , _tag(t) , _extra(e) , _pre(pre) , _post(post) 
	{ }
	/* The above be:
	 *	c  content between tags (empty < /> generated if empty)
	 *	t  the tag <t></t>
	 *	e  tag extra <t e></t> |<t e/>
	 *	pre  pretag insert  pre<t e></t> | pre<t e/>
	 *	post  posttag insert  <t e></t>post | <t e/>post
	 *
	 * 	Why ?
	 *	post for example is good for putting a '\n' 
	 *	outside of <pre></pre> tags
	 */
	
	HtmlTag(HtmlTag ht , string t, string e ,  string pre="" , string post="" ) 
	: string( ht.begin() , ht.end() ) , _tag(t) , _extra(e) , _pre(pre) , _post(post) 
	{ }


	HtmlTag(const HtmlTag& ht)
	: string( ht.begin() , ht.end() )
	{
		_pre = ht._pre;
		_tag = ht._tag;
		_extra = ht._extra;
		_post = ht._post;
	}

	string print() {

		string os(_pre);
		os += "<"; 
		os += _tag;

		if (_extra.length()) {
			os += " ";
			os += _extra ;
		}

		if ( (size()) ) {

			os += ">";
			os += std::string(begin() , end());
			os += "</";
			os += _tag;
			os += ">";
			
		} else {

			os += " />";
		}
		os += _post;

		return os;
		
	}

	std::string content()
	{ return std::string( begin() , end() ); }

	HtmlTag clear_content()
	{ erase( begin() , end() ); return *this; }

	HtmlTag set_tag(string t)
	{ _tag = t; return *this; }
	
	HtmlTag clear_tag() 
	{ _tag = ""; return *this; }

	HtmlTag set_pre(string p)
	{ _pre = p; return *this; }
	
	HtmlTag clear_pre() 
	{ _pre = ""; return *this; }

	HtmlTag set_post(string p) 
	{ _post = p; return *this;}
	
	HtmlTag clear_post() 
	{ _post = ""; return *this; }

	HtmlTag clear_all() 
	{ erase( begin() , end() ) ; _tag="" ; _pre="" ; _post=""; return *this; }

	HtmlTag &operator<<(char* c) 
	{  append(c) ; return *this; }
	
	HtmlTag &operator<<(string s) 
	{  append(s) ; return *this;} 
	
	HtmlTag &operator<<(HtmlTag ht)	
	{ append( ht.print() ); return (*this); }

	friend ostream &operator<<(ostream& os, HtmlTag ht)
		{ os << ht.print(); return os; }

private:
	string _tag;
	string _extra;
	string _pre;
	string _post;
};

#endif
