#include "ObservationSummary.h"
#include <iostream>


ObservationSummary::~ObservationSummary()
{
	/* Need this to stop destructor destroying input data sets with the object */
	_fitsum = NULL;
	_pidx = NULL;

}

ObservationSummary::ObservationSummary(DataSet* fitsum , DataSet* pidx ) 
: PpssummCommon( fitsum , pidx )  
{
	
	guesto();
	
	proposal(); 

	obsrec();

	instrument(); 

	exposure();

}

void ObservationSummary::guesto() {

	std::string tname ("PROPOSAL");
	if ( _fitsum->hasBlock(tname) ) {

		Table* tbl = _fitsum->table(tname);
		
		const std::string want[] = {
			"GONAME"
			,"GOINSTITUTE"
			,"GOADDRLN1"
			,"GOADDRLN2"
			,"GOADDRLN3"
			,"GOADDRLN4"
			,"GOADDRLN5"
			,"GOADDRLN6"
			,"GOEMAIL"
			,""
		};
		const std::string* w = want;

		_notBlank test;
		while ( w->length() ) {
			if ( tbl->hasColumn(*w) ) {
				
				strlist tmp;
				Column* col = tbl->column(*w);
				stringColumn(col , back_inserter(tmp));

				if (tmp.size() && test(tmp[0]) )
					 guesto_data.push_back(tmp[0]);
				
			} else {

				errstr << "Unable to find column "
					<< *w << " in table PROPOSAL of summary file."
					<< Sas::fatal("ppssumm:BADSUMMARYFILE")
				;
					
			}
			w++;
		}
	} else {
		errstr << "Unable to find PROPOSAL table in sumary file" << Sas::fatal("ppssumm:BADSUMMARYFILE");
	}
	std::transform( guesto_data.begin() , guesto_data.end() , guesto_data.begin() , _toString<std::string>() );
	
	
}

void ObservationSummary::proposal() {
	
	std::string tname("PROPOSAL");
	if ( _fitsum->hasBlock(tname) ) {

		Table* tbl = _fitsum->table(tname);
		
		const std::string want[] = {
			"TARGET"
			,"TARGET_RA"
			,"TARGET_DEC"
			,"PROPOSED_DURATION"
			,"ALT_NAMES"
			,"BORESIGHT_RA"
			,"BORESIGHT_DEC"
			,"POS_ANGLE_LC"
			,"POS_ANGLE_UC"
			,"POS_ANGLE_REF"
			,"AO"
			,"SCIENCE_TYPE"
			,""
		};
		const std::string* w = want;

		while ( w->length() ) {

			if ( tbl->hasColumn(*w) ) {

				strlist tmp;
				Column* col = tbl->column(*w);
				stringColumn(col , back_inserter(proposal_data));
				proposal_names.push_back( colName(*w) );

			} else {
				errstr << "Unable to find column "
					<< *w << " in table PROPOSAL of summary file."
					<< Sas::fatal("ppssumm:BADSUMMARYFILE");
			}
			w++;
		}
	} else {
		errstr << "Unable to find PROPOSAL table in sumary file" << Sas::fatal("ppssumm:BADSUMMARYFILE");
	}
	std::transform( proposal_data.begin() , proposal_data.end() , proposal_data.begin() , _toString<std::string>() );

}

void ObservationSummary::obsrec() {
	
	std::string tname("OBSERVAT");
	if ( _fitsum->hasBlock(tname) ) {

		Table* tbl = _fitsum->table(tname);
		
		const std::string want[] = {
			 "OBS_ID"
			,"REVOLUTION"
			,"OBS_START"
			,"OBS_STOP"
			,""
		};
		const std::string* w = want;

		while ( w->length() ) {

			if ( tbl->hasColumn(*w) ) {
				
				Column* col = tbl->column(*w);
				stringColumn(col , back_inserter(obsrec_data));
				obsrec_names.push_back( colName(*w) );

			} else {
				errstr << "Unable to find column "
					<< *w << " in table OBSERVAT of summary file."
					<< Sas::fatal("ppssumm:BADSUMMARYFILE");
					
			}
			w++;
		}
	} else {
		errstr << "Unable to find OBSERVAT table in sumary file" << Sas::fatal("ppssumm:BADSUMMARYFILE");
	}
	std::transform( obsrec_data.begin() , obsrec_data.end() , obsrec_data.begin() , _toString<std::string>() );
	
}

void ObservationSummary::instrument() {


	/* 
	 * We use PpssummCommon::instrument_title and 
	 * PpssummCommon::instrument_table as the 
	 * source for this section
	 */

	/* 
	 * This function is entirely covered by the PpssummCommon data.
	 * It has been left as a hook for future development
	 */

}

void ObservationSummary::exposure() {

	/* 
	 * We use PpssummCommon::exposure_title and 
	 * PpssummCommon::_exposure_tables as the
	 * source for this section.  First concatenate the 
	 * tables before writing out.
	 */

	for (strlist::const_iterator CI = _allInstrument.begin() ; CI != _allInstrument.end() ; CI++ ) {

		keyed_tables::iterator MI;
		MI = _exposure_tables.find(*CI);
		
		if ( MI == _exposure_tables.end() ) continue;
		
		strtable& idata = MI->second;
		if ( ! mergedexp_table.size() ) mergedexp_table.insert( mergedexp_table.end() , idata.size() , strlist() );
		
		
		for( strtable::size_type i = 0 ; i < idata.size() ; i++)
			mergedexp_table[i].insert( mergedexp_table[i].end() , idata[i].begin() , idata[i].end() );

	}

	return;
	
}

HtmlPage ObservationSummary::print()
{
	HtmlPage _page("obspage");
	
	std::string htitle("Observation " + PpssummCommon::_obsid + " Pipeline Processing Summary");
	errstr << "Generating " << htitle << Sas::message(Msg::App, Msg::Verbose);

	const HtmlTag hr("hr");

	/* Top of the page */ 
	_page.title( htitle );

	_page.linkbar("obspage");
	_page._body << hr;
	
	_page.sub_title( PpssummCommon::_sub_title , PpssummCommon::_obsid + " Observation Data Summary File");
	_page._body << hr;

	/* middle of page */

	print_guesto(_page);
	print_proposal(_page); 
	print_obsrec(_page);
	print_instrument(_page); 
	print_exposure(_page);

	/* End of page */

	return _page;
}
void ObservationSummary::print_guesto(HtmlPage& page)
{
	page._body << HtmlTag("h3","","Guest Observer Information");

	HtmlTag pre("pre");
	for ( strlist::const_iterator CI = guesto_data.begin()
		; CI != guesto_data.end()
		; CI++
	) {
		pre << HtmlTag("tt","",*CI,"","\n");
	}
	
	page._body << pre;
	page._body << HtmlTag("hr");

}

void ObservationSummary::print_proposal(HtmlPage& page) 
{
	page._body << HtmlTag("h3","","Proposal Target Information");
	
	unifylength< strlist::iterator > ul(" ");

	ul(proposal_names.begin() , proposal_names.end());

	const std::string sep(" : ");
	HtmlTag pre("pre");
	
	for (strlist::size_type i=0; i< proposal_data.size() ; i++) {
		std::string content(proposal_names[i]);
		content += sep + proposal_data[i];
		pre << HtmlTag("tt","",content,"","\n");
	}

	page._body << pre;
	page._body << HtmlTag("hr");

}

void ObservationSummary::print_obsrec(HtmlPage& page)
{	
	page._body << HtmlTag("h3","","Observation Record");
	
	unifylength< strlist::iterator > ul(" ");
	ul(obsrec_names.begin() , obsrec_names.end());
	ul(obsrec_data.begin() , obsrec_data.end());

	const std::string sep = " : ";
	HtmlTag pre("pre");

	for (unsigned int i = 0; i < obsrec_data.size() ; i++) 
		pre << HtmlTag("tt","",obsrec_names[i] + sep + obsrec_data[i],"","\n")
	;
	
	page._body << pre;
	page._body << HtmlTag("hr");

}

void ObservationSummary::print_instrument(HtmlPage& page)
{
	page._body << HtmlTag("h3","","Instrument Information");
	page.addTable(PpssummCommon::_instrument_title , PpssummCommon::_instrument_table);
	page._body << HtmlTag("hr");

}

void ObservationSummary::print_exposure(HtmlPage& page)
{

	page._body << HtmlTag("h3","","Exposure and Configuration Information");

	strlist exp_title = _exposure_title;
	strlist::iterator titlechng = std::find(exp_title.begin() , exp_title.end() , colName("DURATION") );
	*titlechng = altName("DURATION");

	page.addTable(exp_title , mergedexp_table);
	page._body << HtmlTag("hr");
}

