#ifndef __PPSSUMM_PPSSUMMCOMMON__
#define __PPSSUMM_PPSSUMMCOMMON__

#include <cctype>
#include <algorithm>
#include <functional>
#include <iterator>
#include "ppssumm_typedefs.h"
#include "PpssummStatic.h"
#include "FitsTable.h"
#include "FitsColumn.h"
#include "ExtendFitsColumn.h"
#include "string_algo.h"
#include "STime.h"
#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"

class PpssummCommon
:public PpssummStatic
{
public:
	/* 
	 * This class wraps common data and functions to construct it
	 * This data depends on the input files.
	 */

	/*
	 * fits summary file and product index. Needed by all pages.
	 */
	DataSet* _fitsum;
	DataSet* _pidx;

	PpssummCommon( DataSet* fitsum , DataSet* pidx ) 
	: PpssummStatic()
	{ 
		_fitsum = fitsum;
		_pidx = pidx;
		if (! _constructordone ) {
			dataSimpleValues();
			dataSubTitle();
			dataInstrumentTitles();
			dataInstrumentTable();
			dataExposureTitles();
			dataExposureTables();
			dataProcessedTables();
			_constructordone = true;
		}
	
	} 
	~PpssummCommon(); 

	/* The observation id */
	static std::string _obsid;
	static std::string _procdate;
		
	/* Data used to construct the htmlpage sub title */
	static strlist _sub_title;

	/* Details about each instrument */	
	static strlist _instrument_title;
	static strtable _instrument_table;

	/* Details from all exposures available */
	static strlist _exposure_title;
	static keyed_tables _exposure_tables;

	/* Details about exposures actually processed */
	static keyed_tables _processed_tables;

	/* Extra usefull functions */
	
	std::string dataDataMode(Table* tbl , unsigned int offset) const; 

	/* exposures avaialbe which were actually processed */
	
private: 

	/* These flags control initialization and the constructor */
	static bool _constructordone;
	void dataSimpleValues();  
	void dataSubTitle();  
	void dataInstrumentTitles();
	void dataInstrumentTable();
	void dataExposureTitles();
	void dataExposureTables();
	void dataProcessedTables();

};

#endif
