#include "PpssummStatic.h"

/* define static members */
bool    PpssummStatic::_initdone = false;
strlist PpssummStatic::_allInstrument;
strlist PpssummStatic::_activeInstrument;
strlist PpssummStatic::_anchors;
ssmap   PpssummStatic::_instname;
ssmap   PpssummStatic::_energyband;
ssmap   PpssummStatic::_colname;
ssmap   PpssummStatic::_altname;
ssmap   PpssummStatic::_pageNames;

std::string PpssummStatic::_ppssummVersion;

void PpssummStatic::initVersion() 
{
	Info::Task* self=Info::Task::current();
	PpssummStatic::_ppssummVersion = std::string(self->name()) 
		+ " v" 
		+ std::string(self->version()) 
		+ " [" 
		+ std::string(self->release()) + "]"
	;

}

void PpssummStatic::initPageNames() {
	
	/* Instrument to page name */

	std::string keys[] = {
		"EMOS1"
		,"EMOS2"
		,"EPN"
		,"OM"
		,"RGS1"
		,"RGS2"
		,""
	};

	std::string params[] = {
		"epxpage"
		,"epxpage"
		,"epxpage"
		,"omxpage"
		,"rgxpage"
		,"rgxpage"
		,""
	};

	std::string* k = keys;
	std::string* p = params;

	while ( k->size() && p->size() ) {
	
		PpssummStatic::_pageNames[*k] = stripdir(stringParameter(*p));
		p++;
		k++;
	}
}

void PpssummStatic::initInstrument()
{
	const std::string* p;

	/* Instrument names */
	static const std::string instn[] = {
		"M1" , "EMOS1"
		,"M2" , "EMOS2"
		,"R1" , "RGS1"
		,"R2" , "RGS2"
		,"OM" , "OM"
		,"PN" , "EPN"
		,"EMOS1" , "M1"
		,"EMOS2" , "M2"
		,"RGS1" , "R1"
		,"RGS2" , "R2"
		,"EPN" , "PN"
		,""
	};
	
	PpssummStatic::_instname.clear();
	p=instn;
	while ( p->length() ) PpssummStatic::_instname[*p++] = *p++;

	/* Short Instrument names */
	static const std::string sinst[] = {
		"M1"
		,"M2"
		,"PN"
		,"OM"
		,"R1"
		,"R2"
		,""
	};

	_allInstrument.clear();
	p=sinst;
	while ( p->length() ) _allInstrument.push_back(*p++);

}

void PpssummStatic::initEnergyband()
{
	/* Energy Bands */
	static const std::string EBand[] = {
		 "0" , "0.2 - 12.0"
		,"1" , "0.2 - 0.5"
		,"2" , "0.5 - 2.0"
		,"3" , "2.0 - 4.5"
		,"4" , "4.5 - 7.5"
		,"5" , "7.5 - 12.0"
		,"6" , "0.2  - 2.0"
		,"7" , "2.0 - 12.0"
		,"8" , "0.2 - 12.0"
		,"9" , "0.2 - 2.0"
		,""
	};

	PpssummStatic::_energyband.clear();
	const std::string* p = EBand;
	while ( p->length() ) PpssummStatic::_energyband[*p++] = *p++;

}

void PpssummStatic::initAltname() 
{
	/* 
	 * Alternate key/value pairs used for column headings 
	 * Terminated by an empty string 
	 */
	
	static const std::string altn[] = {
		"DATA_MODE" , "Mode"
		,"DURATION" , "Total Duration"
		,"EPIC_FILTER" , "Filter"
		,"EXP_START" , "Start time"
		,"EXP_STOP" , "Stop time"
		,"EXP_ID" , "ExpID"
		,"EXPID" , "ExpID"
		,"FILTER_WHEEL" , "Position"
		,"FILTER_POTENTIOMETER" , "Position"
		,"INSTRUMENT" , "Inst"
		,"OM_FILTER" , "Filter"
		,"RGSORDER" , "Order" 
		,"SCHEDULED" , "Sched"
		,""
	};

	PpssummStatic::_altname.clear();
	const std::string* p=altn;
	while ( p->length() ) PpssummStatic::_altname[*p++] = *p++;
}

void PpssummStatic::initColname() 
{
	/* 
	 * Alternate key/value pairs used for column headings 
	 * Terminated by an empty string 
	 */

	static const std::string coln[] = {
		"ACTIVE" , "Active"
		,"ACTUAL_START" , "Actual Start"
		,"ACTUAL_END" , "Actual End"
		,"ALT_NAMES" , "Alt Names"
		,"AO" , "AO Number"
		,"AUXILIARY" , "Auxiliary"
		,"BACKGROUND" , "Background"
		,"BAND" , "Band (keV)"
		,"BORESIGHT_DEC" , "Boresight Dec"
		,"BORESIGHT_RA"  , "Boresight RA"
		,"BOX-LOCAL" , "Box-Local"
		,"BOX-MAP" , "Box-Map"
		,"BRIGHT" , "Bright"
		,"CATALOGUE" , "Catalogue"
		,"CONTENT" , "Content"
		,"CROSS-CORRELATI" , "Cross-Correlations"
		,"CROSS-CORRELATION" , "Cross-Correlation"
		,"CROSS-CORRELATIONS" , "Cross-Correlations"
		,"DATAMODE" , "Inst Mode"
		,"DATA_MODE" , "Data Mode"
		,"DEC_PNT" , "Dec"
		,"DESCRIPTIONS" , "Descriptions"
		,"DURATION" , "Duration"
		,"EPIC_FILTER" , "Filter"
		,"EVENT" , "Event"
		,"EXPID" , "Exposure ID"
		,"EXPOSURE" , "Exposure"
		,"EXP_DURATION" , "Duration"
		,"EXP_ID" , "Exposure ID"
		,"EXP_START" ,"Start time"
		,"EXP_STOP" , "Stop time"
		,"FILENAME" , "Filename"
		,"FILTER" , "Filter"
		,"FINDING" , "Finding"
		,"FLATFIELD" , "Flatfield"
		,"FLUCTUATIONS" , "Fluctuations"
		,"HISTOGRAM" , "Histogram"
		,"HISTORY" , "History"
		,"HOUSEKEEPING" , "Housekeeping"
		,"IMAGE" , "Image"
		,"IMAGING" , "Imaging"
		,"INSTRUMENT" , "Instrument"
		,"INTENSITY" , "Intensity"
		,"MESSAGE" , "Message"
		,"MODE" , "Mode"
		,"NAME" , "Filename"
		,"NREXP" , "Exposures"
		,"OBJECT" , "Object"
		,"OBSERVATION" , "Observation"
		,"OBSERVER" , "Observer"
		,"OBS_DURATION" , "Duration"
		,"OBS_ID" , "Observation ID"
		,"OBS_MODE" , "Mode"
		,"OBS_START" , "Start time"
		,"OBS_STOP" , "Stop time"
		,"OCB" , "OCB"
		,"OM_FILTER" , "Filter"
		,"OSW_ID" , "OM Science Window"
		,"PLOT" , "Plot"
		,"POINTED" , "Pointed"
		,"POINTING" , "Pointing"
		,"POSITION" , "Position"
		,"POS_ANGLE_LC" , "Lower Pos Ang"
		,"POS_ANGLE_REF" , "SC Pos Ang"
		,"POS_ANGLE_UC" , "Upper Pos Ang"
		,"PPSVERSION" , "PPS Version"
		,"PRIORITY" , "Priority"
		,"PROCREVISION" , "Proc Revision"
		,"PRODUCT" , "Product"
		,"PROPOSED_DURATION" , "Prop Duration"
		,"RA_PNT" , "RA"
		,"REVOLUTION" , "Revolution"
		,"RGSORDER" , "Order"
		,"RGSORDER" , "Order"
		,"SASVERSION" , "SAS Version"
		,"SinstENCE_TYPE" , "Science Type"
		,"SENSITIVITY" , "SENS"
		,"SLEW" , "Slew"
		,"SOURCE" , "Source"
		,"SPECTRUM" , "Spectrum"
		,"SRC_NUM" , "Source Number"
		,"SUMMARY" , "Summary"
		,"TARGET" ,"Target"
		,"TARGET_DEC" , "Dec"
		,"TARGET_RA" , "RA"
		,"TIMESERIES" , "Timeseries"
		,"TIMING" , "Timing"
		,"TRACKING" , "Tracking"
		,"TRAILED" , "Trailed"
		,"VVFLAGS" , "V & V Flags"
		,""
	};

	PpssummStatic::_colname.clear();
	const std::string* p = coln;
	while ( p->length() )  PpssummStatic::_colname[*p++] = *p++;
}

