#ifndef __PPSSUMM_PPSSUMM_STATIC__
#define __PPSSUMM_PPSSUMM_STATIC__

#include <algorithm>
#include "ppssumm_typedefs.h"
#include "string_algo.h"
#include "Task.h"
#include "param.h"
#include "Tracer.h"
#include "errstr.h"

class PpssummStatic
{
public:
	/* 
	 * This class wraps data which span the different summary pages.
	 *
	 */


	PpssummStatic()
	{ 
		if ( ! _initdone ) PpssummStatic::init();
	} 


	static void init() 
	{
		initVersion();
		initPageNames();
		initInstrument();
		initEnergyband();
		initColname();
		initAltname();
		_initdone = true;

	}

	/* Here are some functions for accessing the data 
	 * contained in this class
	 */

	std::string version() const
	{ return PpssummStatic::_ppssummVersion; }

	std::string band( std::string b  ) const 
	{
		if ( ! b.size() ) return std::string("");

		_toUpper _tu;
		_tu( b );
	
		ssmap::const_iterator found;
		std::string rtn(b);
		found = PpssummStatic::_energyband.find(b);

		if (found != _energyband.end()) rtn = found->second;
		else rtn = std::string(""); 
			

		return rtn;
	}
	
	std::string altName( std::string c  ) const 
	{
		if ( ! c.size() ) return std::string("");
	
		_toUpper _tu;
		_tu( c );

		ssmap::const_iterator found;
		std::string rtn(c);
		found = PpssummStatic::_altname.find(c);

		if (found != _altname.end()) rtn = found->second;
		else rtn = colName(c); 
			

		return rtn;
	}

	std::string colName( std::string c ) const
	{
		if ( ! c.size() ) return std::string("");
	
		_toUpper _tu;
		_tu( c );

		ssmap::const_iterator found;
		std::string rtn(c);
		found = PpssummStatic::_colname.find(c);
		
		if (found != PpssummStatic::_colname.end()) rtn = found->second;
		else rtn = reformat(c);

		return rtn;
	}

	std::string instName( std::string c ) const 
	{
		if ( ! c.size() ) return std::string("");
		
		_toUpper _tu;
		_tu( c );

		ssmap::const_iterator found;
		std::string rtn(c);
		found = PpssummStatic::_instname.find(c);
		
		if (found != PpssummStatic::_instname.end()) rtn = found->second;
		else rtn = reformat(c);

		return rtn;
	}

	bool isActive(std::string s) const
	{
		return std::find( PpssummStatic::_activeInstrument.begin() , PpssummStatic::_activeInstrument.end() , s) != PpssummStatic::_activeInstrument.end();
	
	}

	bool hasAnchor(std::string s) const
	{

		return std::find( PpssummStatic::_anchors.begin() , PpssummStatic::_anchors.end() , s) != PpssummStatic::_anchors.end();

	}

	std::string whichPage(std::string p) 
	{
		if ( ! p.size() ) return std::string("");

		_toUpper _tu;
		_tu( p );

		std::string rtn("");
		ssmap::const_iterator found = PpssummStatic::_pageNames.find(p);

		if (found != PpssummStatic::_pageNames.end()) rtn = found->second;
		else rtn = std::string(""); 

		return rtn;
	}


	/* 
	 * Used to store translations form input names to output text
	 * and comman data used by the above functions 
	 */

	/*
	 * These are public because it is useful to be able to access them
	 * directly.  
	 */
	static strlist _allInstrument;
	static strlist _activeInstrument;
	static strlist _anchors;
	static ssmap _pageNames;

private: 
	/* Task version */
	static std::string _ppssummVersion;

	/* This flags control initialization and the constructor */
	static bool _initdone;

	static ssmap _instname;
	static ssmap _energyband;
	static ssmap _colname;
	static ssmap _altname;

	/* Class init needing done once */
	static void initVersion();
	static void initPageNames();
	static void initInstrument();
	static void initEnergyband();
	static void initColname();
	static void initAltname();

	// This is used sufficiently often it is worthwile putting it here

	std::string reformat(const std::string s) const
	{
		/* 
		 * A fall back function for undefiend name-> column title translations
		 * simply add a '+ to make it obvious.
		 */
		std::string rtn(s);
		return rtn+"+";
		
	}


};

#endif
