#ifndef __PPSSUMMS_SIMPLECOLUMN__
#define __PPSSUMMS_SIMPLECOLUMN__

#include <string>
#include <vector>
#include <ostream>
#include "SAttribute.h"
#include "stripws.h"

#include "Dal.h"

class SColumn : public std::vector<std::string>
{
	// A container for table columns
public:
	SColumn(std::string n="" , std::string c="") 
	: _name(n) , _comment(c)
	{
		clear();
	}

	explicit SColumn(SAttribute sa) 
	: std::vector<std::string>(1,sa.value()) , _name(sa.name()) , _comment(sa.comment())
	{}

	std::string name() const { return _name; }
	SColumn& name(std::string n)
	{ _name = n ; return *this ; }
	
	std::string comment() const { return _comment; }
	SColumn& comment(std::string c)
	{ _comment = c ; return *this ; }

	std::string::size_type width() const
	{
		/* Return length of longest string held in the container */
		std::string::size_type width = 0;
		for(const_iterator CI = begin() ; CI != end() ; CI++) {
			if ( CI->size() > width) width =  CI->size(); 
		}
		return width;
	}

	void pad(unsigned int n , std::string::size_type wdth=0 )
	{
		/* Add n blank rows to a column of width wdth */
		std::string::size_type w = (wdth > width()) ? wdth : width();

		while (n-- && w) std::vector<std::string>::push_back( std::string(w , '\0' ) );
		
	}
	
	void push_back(SAttribute sa) 
	{
		/* Add the _value of an SAttribute to the column */

		std::vector<std::string>::push_back(sa.value()); 
	}

	void insert(SColumn::const_iterator _begin , SColumn::const_iterator _end) 
	{
		while (_begin != _end) {
			std::vector<std::string>::push_back( *_begin++ );
		}
		
	}

	friend std::ostream &operator<<(std::ostream& os , SColumn sc)
	{
		os << "Name/Comment/Length/Width = "
		<< sc.name() << "/" << sc.comment() << "/" << sc.size() << "/" << sc.width() << std::endl;
		return os;

	}
private:
	std::string _name;
	std::string _comment;

};

#endif
