/*
 ppssumm

   Authors:  Duncan John Fyfe - SSC/LUX
         
   Description: Summerise Results of PCMS Processing
 
   Comments: 
 
  $Id: module.cc,v 1.5 2000/10/02 08:30:13 gvacanti Exp $
*/
#include <fstream>
#include <string>
#include <iostream>

#include "getSumFile.h"
#include "ObsSummary.h"

#include "PpssummStatic.h"
#include "PpssummCommon.h"
#include "ObservationSummary.h"
#include "ProductSummary.h"
#include "EpicSummary.h"
#include "OmSummary.h"
#include "RgsSummary.h"
#include "CatSummary.h"

#include "Dal.h"
#include "ppssumm.h"
#include "param.h"
#include "Tracer.h"
#include "errstr.h"

void ppssumm ()
{
PROC
	errstr << "Begin ppssumm" << Sas::message(Msg::App, Msg::Verbose);

	/*
	 * Read observation summary file , convert to a temporary fits file.
	 * Hopefully this will be split into a separate task in the devtrack
	 */

	DataSet* fitsum;
	std::string optname(stringParameter("fitsum"));
	if ( optname.empty() ) 
	{
		optname = tempnam(0,0);
	}
	fitsum = dataSetServer->open( optname.c_str() , DataSet::Create );

	//std::string fitsum = (optname != 0)
	//	? optname
	//	: tempnam(0,0)
	//;

	//DataSet* fitsum = dataSetServer->open( fitsum , DataSet::Temp ); 
	//DataSet* fitsum = dataSetServer->open( optname.c_str() , DataSet::Temp ); 

	std::string inputSummaryFile = getSumFile();
	ObsSummary( inputSummaryFile , fitsum );
	
	/*
	 * Open product index 
	 */
	DataSet* pidx = dataSetServer->open( stringParameter("pindex") , Dal::Read);

	/* initialize common data */
	PpssummStatic::init();
	PpssummCommon::init();


	/* Each page must be created before output for cross-page links to 
	 * be built
	 */

	ObservationSummary obssum( fitsum , pidx );
	ProductSummary ppssum( fitsum , pidx );
	EpicSummary epxsum( fitsum , pidx );
	OmSummary omxsum( fitsum , pidx );
	RgsSummary rgxsum( fitsum , pidx );

	HtmlPage::_procdate = PpssummCommon::_procdate;
	
	std::string fname; 
	std::ofstream out;
		
	fname = stringParameter("obspage");
	out.open(fname.c_str(), std::ios::out);
	if (!out) {

	} else {
		out << obssum;
	}
	out.close();
	
	fname = stringParameter("ppspage");
	out.open(fname.c_str(), std::ios::out);
	if (!out) {

	} else {
		out << ppssum;
	}
	out.close();

	fname = stringParameter("epxpage");
	out.open(fname.c_str(), std::ios::out);
	if (!out) {

	} else {
		out << epxsum;
	}
	out.close();

	fname = stringParameter("omxpage");
	out.open(fname.c_str(), std::ios::out);
	if (!out) {

	} else {
		out << omxsum;
	}
	out.close();
	fname = stringParameter("rgxpage");
	out.open(fname.c_str(), std::ios::out);
	if (!out) {

	} else {
		out << rgxsum;
	}
	out.close();

	std::string catpage = stringParameter("catpage");
	if ( ! catSummaryExists(catpage.c_str()) )
	{
		blankCatSummary(catpage.c_str());
	}
	

	dataSetServer->close(pidx);
	dataSetServer->close(fitsum);

	errstr << "End ppssumm" <<  Sas::message(Msg::App, Msg::Verbose);
ENDPROC
}
