#ifndef __PPSSUMM__STRING_ALGO__
#define __PPSSUMM__STRING_ALGO__
#include "ppssumm_typedefs.h"
#include <string>
#include <cctype>
#include <sstream>
#include <functional>
#include <iterator>
#include "Dal.h"


std::string stripdir( std::string );

std::string truncTime( std::string ); 

template <typename _Ii>
struct unifylength 
{
	/* 
	 * Originally written to pad all strings in a container to 
	 * be of equal width.  Then converted to apply generally to
	 * a container of containers for which an insert function is
	 * defined.
	 *
	 * There is still a little hardwiring to std::string's because
	 * of asymmetry of stdlib functions.
	 *
	 * Padding is to the size of the largest container or _minsize
	 * whicherver is greater.
	 *
	 * _frontinsert => Add padding to the fronto f the container (rightjutify)
	 * _fill 	=> You guessed it.  What to fill with.  Default is container default.
	 */ 

	public:
	typedef typename iterator_traits<_Ii>::value_type __ttype;
	typedef typename __ttype::size_type __tsize;

	__tsize _minsize;
	bool _frontinsert;
	__ttype _fill;
	
	unifylength(__ttype f = iterator_traits<_Ii>::value_type() 
		, __tsize w=0 
		, bool r = false
	) : _minsize(w) , _frontinsert(r) 
	{ _fill = f; }

	__tsize operator()(_Ii _first , _Ii _last)
	{

		/* Length of longest sequence */
		_Ii _tmp = _first;
		while (_tmp != _last) {
			if (_minsize < _tmp->size()) _minsize = _tmp->size();
			_tmp++;
		}
	
		while ( _first != _last ) {

			/* 
			 * We have to insert one at a time via a loop because 
			 * There isn't a std::string::insert( iterator p, size_type n, string s)
			 * to match std::container::insert( iterator p, size_type n, string s)
			 */

			__tsize diff = _minsize - _first->size();

			if ( _frontinsert ) {

				for ( ; diff ; diff--) {
					_first->insert( _first->begin() , _fill.begin() , _fill.end()) ;
				}

			} else {

				for ( ; diff ; diff--) {
					_first->insert( _first->end()  , _fill.begin() , _fill.end()) ;
				}
			}


			_first++;

		}

		return _minsize;
	}
};


/*
 * I know there is a toString in stingutil.h but it doesn't work
 * for bool8 or DString.  It returns ^A (true) and ^@ (false) 
 * and leaves trailing ^@ on the end of strings.
 */


/* 
 * WARNING
 * 
 * <string> with g++ 2.95.3 #define 's _C to be 32
 * Don't use it ! Especially not as a template parameter.
 * Aaaaaaaaaaaaaaaaaaaaargh!
 * 
 */


template< typename _T >
struct _toString : unary_function< _T , std::string > 
{
	public:
	int _prec;
	_toString(int _p=0) : _prec(_p) {};
	std::string operator()( const _T& _v ) const 
	{
		std::stringstream s;
		if ( _prec > 0 ) s.precision(_prec);
		s << _v;
		return s.str();
		
	}
};

template<>
struct _toString< std::string > : unary_function< std::string , std::string > 
{
	public:
	std::string operator()( const std::string& _v ) const 
	{
		const char n='\0';
		return _v.substr(0,_v.find_first_of(n));
	}
};

template<>
struct _toString< bool8 > : unary_function< bool8 , std::string > 
{
	public:
	std::string _True, _False;
	_toString(char* pT = "Y" , char* pF ="N" ) : _True(pT) , _False(pF) {}
	std::string operator()( const bool8& _v ) const 
	{
		return _v ? _True : _False ;
	}
};


struct _notBlank : unary_function< std::string  , bool  > 
{
	/* 
	 * Simple function for use with stl algorithms.
	 * Return true if the input string has members 
	 * which are not members of _ignore.
	 */
	public:
	std::string _ignore;
	_notBlank( std::string ign = " \t\n" ) : _ignore(ign) {}
	bool operator()( const std::string s ) const 
	{

		bool rtn;
		if ( s.empty() ) rtn = false;
		else if (_ignore.length()) rtn =  (s.find_first_not_of(_ignore) != std::string::npos) ;
		else rtn = true;

		return rtn;
	}
};

struct _toUpper : unary_function< std::string&  , std::string&  > 
{
	/* 
	 * Wrapper for std::toupper for strings
	 */
	public:
	//_toUpper() {}
	std::string& operator()(std::string& s) const
	{
		for( std::string::iterator i = s.begin()  
				; i != s.end()
				; i++
		) *i = static_cast<char>( std::toupper( static_cast<int>( *i ) ) );	

		return s;

	}


};

struct _toLower : unary_function< std::string&  , std::string&  > 
{
	/* 
	 * Wrapper for std::tolower for strings
	 */
	public:
	//_toLower() {}
	std::string& operator()(std::string& s) const
	{
		for( std::string::iterator i = s.begin()  
				; i != s.end()
				; i++
		) *i = static_cast<char>( std::tolower( static_cast<int>( *i ) ) );	

		return s;

	}


};

#endif
